var key = "BB30F27124A66869BA630F2CB7AE7316";
var CMD_INSERT_SUCCESS = "SQL INSERT SUCCESS";

/**
 *  Perform a login.
 *  @return access level (-2 is failure)
 */

function db_doLogin(user, pass)
{
    $.cookie('User', user);
    $.cookie('Hash', pass);
    $.cookie('Password', pass);
    $.cookie('SessionCode', "0");
   
    var cmdUrl = "/scripts/select.dll?function=LOGIN&sessioncode=" + db_getSessionCode();
    
    var response = $.ajax(
    { 
        type: "POST",
        url: cmdUrl,
        async: false
    }).responseText;

    var responseXML = $.parseXML(response);

    var sessionCode = $(responseXML).find('SessionCode').text();
    $.cookie('SessionCode', sessionCode);
    var accessLevel = $(responseXML).find('AccessLevel').text();

    $.cookie('AccessLevel', accessLevel);
    var hashPass = db_hash("", "", pass, "");

    $.cookie('Hash', hashPass);

    // now return the access level
    return accessLevel;
}

//////////////// BASIC QUERY FUNCTIONS ///////////////

function db_executeQuery(command, cb)
{
    // escape the first part of the URI - we won't be passing multiple params after the sql= one, just the session code.
    var cmdUrl = encodeURI("/scripts/select.dll?function=SELECT&sql=" + command) + "&sessioncode=" + db_getSessionCode();
    return $.post(cmdUrl, function(data) { db_checkLoggedIn(data, cb)});
}

/*
 *  Don't perform a select - just do whatever function is passed.  Used for GETTIME, etc.
 */
function db_execute(command, cb)
{
    var cmdUrl = "/scripts/select.dll?function=" + command + "&sessioncode=" + db_getSessionCode();
    return $.post(cmdUrl, function(data) { db_checkLoggedIn(data, cb)});
}

/**
 *  Check for AccessLevel < 0.  If cb is passed, it's part of an asynchronous
 *  call and the given callback should be used on success.  If cb isn't passed, it's
 *  part of a synchronous call and the data passed in can just be returned.
 */
function db_checkLoggedIn(data, cb)
{
    var accessLevel = $(data).find("AccessLevel").text();
    if (accessLevel < 0)
    {
        // instead of executing the callback, redirect to the login page
        alert("Your session has expired, please log in again.");
        window.location = "Login.html";
        return null;
    }
    else
    {
        if (arguments.length == 2)
        {
            // present the data to the original response callback
            cb(data);
        }
        else
        {
            // return the data
            return data;
        }
    }
}

function db_process_special_chars(data)
{
    // replace all \r\n with actual cr-lf
    data = data.replace(/\\r/g, "\r");
    data = data.replace(/\\n/g, "\n");

    return data;
}

/**
 *  Execute the given select query synchronously.  Use sparingly.
 *  Don't have to pass a command - used for GETTIME, etc.
 */
function db_executeQueryTypeSync(cmdType, command)
{
    var tmpURL = "/scripts/select.dll?function=" + cmdType;
    if (arguments.length == 2)
    {
        // we've got some sql to pass.  Need to encode the command part as it may contain &
        // and other characters that will mess up otherwise
        tmpURL += "&sql=" + encodeURIComponent(command);
    }
    
    var cmdUrl = tmpURL + "&sessioncode=" + db_getSessionCode();
    
    var response = $.ajax(
    { 
        type: "POST",
        url: cmdUrl,
        async: false
    }).responseText;

    if (cmdType == "SELECT")
    {
        // now allow any special character processing
        response = db_process_special_chars(response);
 
        // we'll treat this as an XML fragment 
        var xmlResponse = $.parseXML(response);
        return db_checkLoggedIn(xmlResponse);
    }
    else
    {
        // might just contain SQL INSERT SUCCESS or similar
        // TODO: check access level for this too
        return response;
    }
}

/**
 *  Replace a single quote with a double one
 */
function db_escape_quotes(args)
{
    for (var i = 0; i < args.length; i++)
    {
        // only look at strings
        if (typeof args[i] === "string")
        {
            args[i] = args[i].replace(/\'/g, "\'\'");
        }
    }
    return;
}

function db_sys_param_update(codeName, newValue)
{
    var cmd = "UPDATE TBL_SC_SYSTEM_PARAMETERS SET Value=" + newValue + " WHERE codeName='" + codeName + "'";
    var result = db_executeQueryTypeSync("UPDATE", cmd);
    return result.indexOf(CMD_INSERT_SUCCESS) != -1;
}

function db_sys_param_update_by_id(id, newValue)
{
    var cmd = "UPDATE TBL_SC_SYSTEM_PARAMETERS SET Value=" + newValue + " WHERE id=" + id;
    var result = db_executeQueryTypeSync("UPDATE", cmd);
    return result.indexOf(CMD_INSERT_SUCCESS) != -1;
}

function db_sys_param_get(codeName)
{
    var cmd = "SELECT Value FROM TBL_SC_SYSTEM_PARAMETERS WHERE codeName='" + codeName + "'";
    var result = db_executeQueryTypeSync("SELECT", cmd);
    return parseInt($(result).find("Value").text(), 10);
}

function db_sys_param_get_by_id(id)
{
    var cmd = "SELECT Value FROM TBL_SC_SYSTEM_PARAMETERS WHERE id=" + id;
    var result = db_executeQueryTypeSync("SELECT", cmd);
    return parseInt($(result).find("Value").text(), 10);
}

function db_tcoReceiverSave(id, description, present, pollable)
{
    // handle any single-quotes
    db_escape_quotes(arguments);
    
    var cmd = "UPDATE TBL_RECEIVER SET Location_Description='" + description + "', Present=" + present + ", Pollable=" + pollable + " WHERE ID=" + id;
    var result = db_executeQueryTypeSync("UPDATE", cmd);
    return result.indexOf(CMD_INSERT_SUCCESS) != -1;
}


/**
 *  Similar to the other audit info functions, but with customerCode, siteCode and unitCode, and saved into its own table
 */
function db_updateMidasInfo(customerCode, siteCode, unitCode, time, sunday, monday, tuesday, wednesday, thursday, friday, saturday, logDirectory, ftpDirectory)
{
    // handle any single-quotes
    db_escape_quotes(arguments);
    
    var updateCmd = "UPDATE TBL_MIDAS_INFORMATION SET ";
    updateCmd += "CustomerCode=" + customerCode + ", SiteCode=" + siteCode + ", UnitCode=" + unitCode + ", Time='" + time + "', Sunday=" + sunday + ", Monday=" + 
        monday + ", Tuesday=" + tuesday + ", Wednesday=" + wednesday + ", Thursday=" + thursday +
        ", Friday=" + friday + ", Saturday=" + saturday + ", logDirectory='" + logDirectory + "', ftpDirectory='" + ftpDirectory + "'";
   
    updateCmd += " WHERE ID=1";

    var result = db_executeQueryTypeSync("UPDATE", updateCmd);
    return result.indexOf(CMD_INSERT_SUCCESS) != -1;
}

function db_addUser(username, password, cb)
{
    // handle any single-quotes
    db_escape_quotes(arguments);
    
    var hash1 = db_hash(key, username, password, "0");
    var hash2 = db_hash("", "", password, "");
    var cmd = "ADDUSER&username=" + username + "&hash=" + hash1 + "&2ndHash=" + hash2;
    db_execute(cmd, cb);
}

function db_removeUser(username, cb)
{
    // handle any single-quotes
    db_escape_quotes(arguments);
    
    var cmd = "DELETEUSER&username=" + username;
    db_execute(cmd, cb);
}

/*
 *  Take the first 8 characters of the given password and check that it matches the one used to login.
 *  TODO: XXX consider what happens in a normal, non-Service/Dev login
 */
function db_checkPass(userPass)
{
    return userPass.substring(0,8) === $.cookie('Password');
}

function db_changeLogin(username, password, cb)
{
    // handle any single-quotes
    db_escape_quotes(arguments);
    
    var hash1 = db_hash(key, username, password, "0");
    var hash2 = db_hash("", "", password, "");
    var cmd = "CHANGELOGIN&username=" + username + "&hash=" + hash1 + "&2ndHash=" + hash2;
    db_execute(cmd, cb);
}

function db_count(table, cb) 
{
    // count rows in a table
    var query = "SELECT count(*) as numRows from " + table;
    if (arguments.length > 1)
    {
        // use async callback
        db_executeQuery(query, cb);
    }
    else
    {
        // synchronous request
        var result = db_executeQueryTypeSync("SELECT", query);
        var count = parseInt($(result).find("numRows").text(), 10);
        if (isNaN(count))
        {
            return 0;
        }
        else
        {
            return count;
        }

        return rowCount;
    }
}

function db_sysSummary(cb)
{
    // note that this is a query on a virtual table - the select.dll treats it specially and
    // requires the exact string below
    var query = "Select * from SYS_SUMMARY";
    db_executeQuery(query, cb);
}

function db_site_mode_set(cgID)
{
    var cmd = "SETSITEMODE&cgID=" + cgID;
    var response = db_executeQueryTypeSync(cmd);
    var xmlResponse = $.parseXML(response);
    return xmlResponse;
}

function db_getSessionCode()
{
    var user = $.cookie('User');
    var hashPass = $.cookie('Hash');
    var sessionCode = $.cookie('SessionCode');
    return db_hash(key, user, hashPass, sessionCode);
}

function db_hash(key, user, pass, session)
{
    // do an MD5 checksum of key+user+pass+session bytes
    var str = key + user + pass + session;
    return hex_md5(str);
}

function db_vox_settings_update(ID, warScale, warThresh, resScale, resThresh, sideTone)
{
    var updateCmd = "UPDATE TBL_VOX_SETTINGS SET wardenScale=" + warScale + ", wardenThresh=" + warThresh + 
        ", resScale=" + resScale + ", resThresh=" + resThresh + ", sideTone=" + sideTone + " WHERE ID=" + ID;
      
    var result = db_executeQueryTypeSync("UPDATE", updateCmd);
    return result.indexOf(CMD_INSERT_SUCCESS) != -1;   
}

function db_matrix_gains_update(ID, name, tel1out, tel1in, tel2out, tel2in, tel3out, tel3in, cc1out, cc1in, cc2out, cc2in, sysTones, speechOut, tableType)
{
    // handle any single-quotes
    db_escape_quotes(arguments);
    
    if ( tableType==0)
    {
        updateCmd = "UPDATE TBL_MATRIX_GAINS";
    }
    else
    {
        updateCmd = "UPDATE TBL_MATRIX_GAINS_X300";
    } 

    updateCmd = updateCmd + " SET name='" + name + "', tel1_out=" + tel1out + ", tel1_in=" + tel1in + ", tel2_out=" + tel2out +
        ", tel2_in=" + tel2in + ", tel3_out=" + tel3out + ", tel3_in=" + tel3in + ", cc1_out=" + cc1out + ", cc1_in=" + cc1in + 
        ", cc2_out=" + cc2out + ", cc2_in=" + cc2in + ", sysTones=" + sysTones + ", spch_out=" + speechOut +
        " WHERE ID=" + ID;
      
    var result = db_executeQueryTypeSync("UPDATE", updateCmd);
    return result.indexOf(CMD_INSERT_SUCCESS) != -1;   
}

function db_force_cim_eeprom_settings_to_default()
{
    // Clearing the site info parameter CIMVersion forces a reset to default CIM EEPROM settings on system restart
    var updateCmd = "UPDATE TBL_SITE_INFORMATION SET CIMVersion='' WHERE ID=1";
    var result = db_executeQueryTypeSync("UPDATE", updateCmd);
    return result.indexOf(CMD_INSERT_SUCCESS) != -1;   
}

function db_incoming_numbers_update(ID, description, incomingTelephone, numberType)
{
    // handle any single-quotes
    db_escape_quotes(arguments);
    
    var updateCmd = "UPDATE TBL_INCOMING_TELEPHONE_NUMBERS SET Description='" + description + "', Incoming_Telephone='" + 
        incomingTelephone + "', Number_Type=" + numberType + " WHERE ID=" + ID;
        
    var result = db_executeQueryTypeSync("UPDATE", updateCmd);
    return result.indexOf(CMD_INSERT_SUCCESS) != -1;
}

/**
 *  Update an existing PBX trunk configuration
 */
function db_pbx_trunk_update(id, description, number, trunkType, trunkName, accountName)
{
	// handle any single-quotes
	db_escape_quotes(arguments);

	// update the PBX trunk row with new description/number
	var cmd = "UPDATE TBL_PBX_TRUNKS SET Description='" + description + "', Telephone_Number='" + number + "', trunkType=" + trunkType + 
		", sipTrunkName='" + trunkName + "', sipTrunkAccountName='" + accountName + "' WHERE ID=" + id;
		
    var result = db_executeQueryTypeSync("UPDATE", cmd);
    var success = (result.indexOf(CMD_INSERT_SUCCESS) != -1);
	return success;
}

function db_pbx_trunk_add(id, description, number, trunkType, trunkName, accountName)
{
	// handle any single-quotes
	db_escape_quotes(arguments);

	// update the PBX trunk row with new description/number
	var cmd = "INSERT INTO TBL_PBX_TRUNKS (ID, Description, Telephone_Number, fxoSectionName, fxoResourceName, trunkType, sipTrunkName, sipTrunkAccountName) VALUES (" +
		id + ",'" + description + "', '" + number + "', '', '', " + trunkType + ", '" + trunkName + "', '" + accountName + "')";
		
    var result = db_executeQueryTypeSync("UPDATE", cmd);
    var success = (result.indexOf(CMD_INSERT_SUCCESS) != -1);
	return success;
}

/**
 *  Remove all existing PBX trunk numbers and replace them with the given descriptions and numbers.
 */
function db_pbx_trunks_replace(descriptions, numbers)
{
    // handle any single-quotes
    db_escape_quotes(arguments);
    
	// remove all existing
	var cmd = "DELETE FROM TBL_PBX_TRUNKS";
    var result = db_executeQueryTypeSync("UPDATE", cmd);
    var success = (result.indexOf(CMD_INSERT_SUCCESS) != -1);
    
    if (success)
	{
		// add entries anew
		for (var i = 0; i < descriptions.length; i++)
		{
			var insertCmd = "INSERT INTO TBL_PBX_TRUNKS (ID, Description, Telephone_Number) VALUES (" + (i + 1) + ", '" + descriptions[i] + "', '" + numbers[i] + "')";
	        result = db_executeQueryTypeSync("INSERT", insertCmd);
        
			if (result.indexOf(CMD_INSERT_SUCCESS) == -1)
			{
				success = false;
				break;
			}
		}
	}
	
	return success;
}	

function db_quick_dial_update(ID, description, telNumber, telInterface, numberType)
{
    // handle any single-quotes
    db_escape_quotes(arguments);
    
    var updateCmd = "UPDATE TBL_SCHEME_MANAGER_QUICK_DIALS SET Description='" + description + "', telephone_number='" + 
        telNumber + "', telephone_interface=" + telInterface + ", Number_Type=" + numberType + " WHERE ID=" + ID;
        
    var result = db_executeQueryTypeSync("UPDATE", updateCmd);
    return result.indexOf(CMD_INSERT_SUCCESS) != -1;
}

function db_gsm_param_update(codeName, paramVal)
{
    var updateCmd = "UPDATE TBL_GSM_PARAMETERS SET value='" + paramVal + "' WHERE codeName='" + codeName + "'";
    var result = db_executeQueryTypeSync("UPDATE", updateCmd);
    return result.indexOf(CMD_INSERT_SUCCESS) != -1;
}


function db_strip_telephone_number(telNum)
{
    // strip the telNum string to just digits
    var strippedNum = "";
    var i;
    for (i = 0; i < telNum.length; i++)
    {
        var ch = telNum.charAt(i);
        if (ch >= "0" && ch <= "9")
        {
            strippedNum+= ch;
        }
    }
    return strippedNum;
}

function db_pbx_ring_in_update(configId, cadenceId, size, matches, expiry, deviation, part0, part1, part2, part3,
    part4, part5, part6, part7, part8, part9)
{
    var updateCmd = "UPDATE TBL_TONES_RING_IN SET size=" + size + ", matches=" + matches + ", expiry=" + expiry +
        ", deviation=" + deviation + ", part0=" + part0 + ", part1=" + part1 + ", part2=" + part2 + ", part3=" + part3 +
        ", part4=" + part4 + ", part5=" + part5 + ", part6=" + part6 + ", part7=" + part7 + ", part8=" + part8 + ", part9=" + part9;
    
    updateCmd += " WHERE configId=" + configId + " AND cadenceId=" + cadenceId;
    
    var result = db_executeQueryTypeSync("UPDATE", updateCmd);
    return result.indexOf(CMD_INSERT_SUCCESS) != -1;    
}

function db_pbx_config_update(ID, tonesHexString, cadences)
{
    var updateCmd = "UPDATE TBL_TONES_PROGRESS SET tones='" + tonesHexString + "'";
    for (var i = 0; i < cadences.length; i++)
    {
        updateCmd += ", cad" + i + "='" + cadences[i] + "'";
    }
    
    updateCmd += " WHERE ID=" + ID;
    var result = db_executeQueryTypeSync("UPDATE", updateCmd);
    return result.indexOf(CMD_INSERT_SUCCESS) != -1;    
}